/*

Servo PIC

Designed by Shane Tolmie, Andrew Dixon and Shane Buckham Jan 99.
Common header file for servo control for the Airborne Control Systems (ACS) flight stabilisation system.

Compiled with Hitech-C v7.82

Usage: #include in all ".c" files in project along with "pic.h"

*/

//warning about #if statements: if any of the arguments are spelled wrong or
//unrecognised by the compiler, it will not generate a warning, but not include code

#ifndef ALWAYS_H
#define ALWAYS_H

#define PIC_CLK 8

#define us_to_ticks(x)	(x*(PIC_CLK/4)/(TMR1_PRESCALE))		//converting microseconds to tmr1 clock ticks
#define ticks_to_us(x)	((x*4/PIC_CLK)*(TMR1_PRESCALE))		//converting tmr1 clock ticks to microseconds

/*

timeouts:

C code for testing with ints:

			unsigned int timeout;
			timeout=4000;
			PORT_DIRECTION=OUTPUT;
			while(1)
			{
				PORT=1;
				timeout=8000;
				while(timeout-- >= 1);	//60ms @ 8Mhz, opt on, 72ms @ 8Mhz, opt off
				PORT=0;
			}

Time taken:	optimisations on:		16cyc/number loop, 8us @ 8Mhz
			optimisations off:		18cyc/number loop, 9us @ 8Mhz
			with extra check ie:	&& (RB7==1), +3cyc/number loop, +1.5us @ 8Mhz

C code for testing with chars:

			similar to above

Time taken:	optimisations on:		9cyc/number loop, 4.5us @ 8Mhz
			with extra check ie:	&& (RB7==1), +3cyc/number loop, +1.5us @ 8Mhz

Formula:	rough timeout value = (<us desired>/<cycles per loop>) * (PIC_CLK/4.0)

To use:		//for max  timeout of 1147us @ 8Mhz
			#define LOOP_CYCLES_CHAR	9					//how many cycles per loop, optimizations on
			#define timeout_char_us(x)	(unsigned char)((x/LOOP_CYCLES_CHAR)*(PIC_CLK/4.0))
			unsigned char timeout;
			timeout=timeout_char_us(1147);						//max timeout allowed @ 8Mhz, 573us @ 16Mhz
			while((timeout-- >= 1) && (<extra condition>));	//wait

To use:		//for max 491512us, half sec timeout @ 8Mhz
			#define LOOP_CYCLES_INT		16					//how many cycles per loop, optimizations on
			#define timeout_int_us(x)	(unsigned int)((x+/LOOP_CYCLES_INT)*(PIC_CLK/4.0))
			unsigned int timeout;
			timeout=timeout_int_us(491512);						//max timeout allowed @ 8Mhz
			while((timeout-- >= 1) && (<extra condition>));	//wait
*/
#define LOOP_CYCLES_CHAR	9							//how many cycles per loop, optimizations on
#define timeout_char_us(x)	(unsigned char)((x/LOOP_CYCLES_CHAR)*(PIC_CLK/4.0))

#define LOOP_CYCLES_INT		16							//how many cycles per loop, optimizations on
#define timeout_int_us(x)	(unsigned int)((x/LOOP_CYCLES_INT)*(PIC_CLK/4.0))

#define THE_BEER_IS_PLENTIFUL_AND_THE_PARTY_SWINGING TRUE

//these two structures allow access to 2 byte word, high and low bytes of variable
//declaration:	union wordtype x;
//usage: 		x.word=0xABCD; x.byte.high=0xAB; x.byte.low=0xCD;
//				x.part.bit15=1; (msb), x.part.bit0=1; (lsb) 
//declaration:	union chartype x;
//usage: 		x.byte=0xAB;
//				x.part.bit7=1; (msb), x.part.bit0=1; (lsb) 


struct sixteen_bits
{
	unsigned char bit0	:1;
	unsigned char bit1	:1;
	unsigned char bit2	:1;
	unsigned char bit3	:1;
	unsigned char bit4	:1;
	unsigned char bit5	:1;
	unsigned char bit6	:1;
	unsigned char bit7	:1;
	unsigned char bit8	:1;
	unsigned char bit9	:1;
	unsigned char bit10	:1;
	unsigned char bit11	:1;
	unsigned char bit12	:1;
	unsigned char bit13	:1;
	unsigned char bit14	:1;
	unsigned char bit15	:1;
};

struct eight_bits
{
	unsigned char bit0	:1;
	unsigned char bit1	:1;
	unsigned char bit2	:1;
	unsigned char bit3	:1;
	unsigned char bit4	:1;
	unsigned char bit5	:1;
	unsigned char bit6	:1;
	unsigned char bit7	:1;
};

struct two_bytes
{
	unsigned char low;
	unsigned char high;
};

union wordtype
{
	unsigned int word;
	struct two_bytes byte;
	struct sixteen_bits part;
};

union chartype
{
	unsigned char byte;
	struct eight_bits part;
};

//defines
#define INPUT 1		//port directions
#define OUTPUT 0
#define TRUE 1
#define FALSE 0
#define	HIGH 1
#define	LOW 0

#define b asm("nop")		//convenient point for breakpoint (debugging)
#define l while(1)			//loop for ever (debugging)

#if (PIC_CLK == 8)
	#define dly125n please remove; for 32Mhz+ only
	#define dly250n please remove; for 16Mhz+ only
	#define dly500n asm("nop")
	#define dly1u dly500n;dly500n
	#define dly2u dly1u;dly1u
#endif

#if (PIC_CLK == 16)
	#define dly125n please remove; for 32Mhz+ only
	#define dly250n asm("nop")
	#define dly500n dly250n;dly250n
	#define dly1u dly500n;dly500n
	#define dly2u dly1u;dly1u
#endif

#if (PIC_CLK == 32)
	#define dly125n asm("nop")
	#define dly250n dly125n;dly125n
	#define dly500n dly250n;dly250n
	#define dly1u dly500n;dly500n
	#define dly2u dly1u;dly1u
#endif

#if ((PIC_CLK != 8) && (PIC_CLK != 16) && (PIC_CLK != 32))
	please define pic_clk please
#endif

//delay routine

//#if PIC_CLK == 8
	#define DelayDivisor 2
	#define WaitFor1Us asm("nop")
	#define Jumpback asm("goto $ - 2")
//#endif

/*
#if PIC_CLK == 16
	#define DelayDivisor 1
	#define WaitFor1Us asm("nop")
	#define Jumpback asm("goto $ - 2")
#endif

#if PIC_CLK == 32
	#define DelayDivisor 1
	#define WaitFor1Us asm("nop"); asm("nop"); asm("nop"); asm("nop"); asm("nop")
	#define Jumpback asm("goto $ - 6")
#endif

*/

#if ((PIC_CLK != 8) && (PIC_CLK != 16) && (PIC_CLK != 32))
	must have pic_clk defined
#endif

//extern unsigned char delayus_variable;


#define DelayUs(x) { \
			delayus_variable=x/DelayDivisor; \
			WaitFor1Us; } \
			asm("decfsz _delayus_variable,f"); \
			Jumpback;
			
#endif

//see AM576.  If interrupt occurs just when gie gets set to zero, it won't be cleared

#define	gie_on		GIE=1
#define gie_off		while(GIE==1) GIE=0
